predict.lfpca <-
function(object, subject=NULL,time=NULL,visit=NULL,log="predict.log",...){

    ## inputs: result: lfpca, subject:numeric.vector,
    ## time:numeric.vector,log:string file path
    ## check subj, time
    ## check if subject, time in result: type-seg,track
    ## for each obs, get Y, Yhat
    ## get no of missing in track
    ## % of missing in segs
    ## give R2: SSM/SST (for missing data)
    ## return: list:
    ## predictions: matrix Y
    ## predict without subject or time
    ## subject=NULL; time=NULL

    model <- object
    ## checking for inputs, error, warnings
    if(class(model) != "lfpca"){stop("Model is not from LFPCA()")}
    lent <- length(time);lens <- length(subject);
    lenv <- length(visit)
    ## both or none of subj, and visit
    if((lens>0 & lent==0) || (lens==0 & lent>0)){
        stop("Either specify both subject and time,",
             "or keep both NULL")
    }else if(lent==0 & lens ==0){
        self <- TRUE
    }else if(lent>0 & lens >0){
        self <- FALSE
        if (lens != lent){
            stop("The number of rows in time vector needs to agree",
                 "with the length of the subject vector")
        }                           #if for length
        if(lenv > 0 & lenv !=lent){
            stop("The number of rows in visit vector needs to agree",
                 "with the length of the subject and time vector")
        }

    }                           #else if: self-false

    if(self){
        ##cat("Estimating all the functions in the model\n")
    }else{
        ##cat("Function received",lens,"subjects",lent,
        ##    "times and",lenv,"visits\n\n")
    }

    ## some global variables
    res <- model;
    D <- ncol(res$Y); n <- nrow(res$Y)
    d.vec <- rep(1:D, each = n); time.vec <- rep(res$time, D)
    uniqSub <- unique(res$subject[order(res$subject)])
    J <- sapply(uniqSub,               # number of visits for each res$subject
                function(sub){sum(res$subject == sub)})
    I <- length(J)           # number of subjects
    inds <- rep(1:I,J)  # index instead of subject id, repeated for visits
    subjInd <- as.data.frame(cbind(res$subject, inds));
    names(subjInd) <- c("subject","inds");
    class(subjInd$inds) <- "numeric"
    subject <- as.numeric(subject);visit <- as.numeric(visit);
    time <- as.numeric(time)

    ## the overall mean surface
    term0 <- matrix(predict(res$eta, newdata=as.data.frame(cbind(d.vec,time.vec))),n,D) #\eta(d,T)
    term1 <- res$xi %*% t(res$phi.0)                          #X_{i,0)(d), second argument talks of the level
    term2 <- res$xi %*% t(res$phi.1)                          #X_{i,1)(d)
    term3 <- res$zeta %*% t(res$phi.U)                        #U_{ij}(d)

    if(self){                           #for the whole dataset
        YHat <- matrix(NA,nrow(res$Y),D)
        for(i in 1:nrow(res$Y)){
            ## term0:
            YHat[i,] <- term0[i,] + term1[inds[i],] +
                term2[inds[i],]*res$time[i] + term3[i,]
        }                               #for ends


    }else{ ## for specific subjects
        n <- lens; #or lent
        d.vec <- rep(1:D, each = n); time.vec <- rep(time, D)
        term0 <- matrix(predict(res$eta,
                                newdata=as.data.frame(cbind(d.vec,time.vec))),n,D)#eta
        ## need to know which subject is which
        ## reference for subVec is uniqSub, here we want subject to be numeric
        subVec <- unlist(sapply(subject, function(s) which(s == uniqSub)))
        YHat <- matrix(NA,n,D)
        ## going in for segment reconstruction
        if(lenv >0){ ## subject and visit come from the same dat.
            subChar <- as.character(subject)
            wrongVis <- sapply((J[subChar] - visit), function(x) x<0)
            if(sum(wrongVis) > 0){
                cat("Visit no supplied cant be greater than the no. of visits",
                    "observed\n")
                stop("Visit information does not agree with",
                     " the number observed in the study")
            }
            ## need to know which visit of which subject, for term3
            obsVec <- sapply(1:length(subject), function(i){
                prevObs <- sum(J[0:(subVec[i]-1)]) # observation till this subject
                obs <- prevObs + visit[i]           #obs in this subject
            })
            for(i in 1:n){
                YHat[i,] <- term0[i,] + term1[subVec[i],] +
                    term2[subVec[i],]*time[i] + term3[obsVec[i],]
            }## for loop
        }else{                          #for lenv
            for(i in 1:n){
                YHat[i,] <- term0[i,] + term1[subVec[i],] +
                    term2[subVec[i],]*time[i]
            }## for loop
        }##for missing track
    }#else for new data
    results <- list("YHat"=YHat)
    class(results) <- "predict.lfpca"
    return(results)
}                                       #lfpca.predict

